@@ -1,5 +1,5 @@
 from __future__ import annotations
-from typing import Dict, Generic, Iterable, Iterator, Union, List, Set, Optional, Tuple, Type, TypeVar, Callable, overload
+from typing import Dict, Generic, Iterable, Iterator, Union, List, Set, Optional, Tuple, Type, TypeVar, Callable, overload, Literal
 
 T = TypeVar("T")
 R = TypeVar("R")
@@ -799,6 +799,96 @@ class Sequence(Generic[T], Iterable[T]):
         """
         return value in self
     
+
+    def element_at(self, index: int) -> T:
+        """
+         Returns the element at the specified [index] in the Sequence.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).element_at(1)
+        2
+
+         Example 2:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).element_at(3)
+        Traceback (most recent call last):
+        ...
+        IndexError: Index 3 out of range
+        """
+        return self.element_at_or_else(index, lambda index: throw(IndexError(f'Index {index} out of range')))
+    
+    @overload
+    def element_at_or_else(self, index: int, default: T) -> T:
+        """
+         Returns the element at the specified [index] in the Sequence or the [default] value if the index is out of bounds.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).element_at_or_else(1, 'default')
+        2
+        >>> it(lst).element_at_or_else(4, lambda x: 'default')
+        'default'
+
+        """
+        ...
+    
+    @overload
+    def element_at_or_else(self, index: int, default: None) -> Optional[T]:
+        ...
+    @overload
+    def element_at_or_else(self, index: int, default: Callable[[int], T]) -> T:
+        ...
+    def element_at_or_else(self, index: int, default: Union[Callable[[int], T], T, None]) -> Union[Optional[T], T]:
+        """
+         Returns the element at the specified [index] in the Sequence or the [default] value if the index is out of bounds.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).element_at_or_else(1, lambda x: 'default')
+        2
+        >>> it(lst).element_at_or_else(4, lambda x: 'default')
+        'default'
+
+        """
+        if (index < 0):
+            return default(index) if callable(default) else default
+        
+        for i, e in enumerate(self):
+            if i == index:
+                return e
+        return default(index) if callable(default) else default
+    
+
+    def element_at_or_default(self, index: int, default: T) -> T:
+        """
+         Returns the element at the specified [index] in the Sequence or the [default] value if the index is out of bounds.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).element_at_or_default(1, 'default')
+        2
+        >>> it(lst).element_at_or_default(4, 'default')
+        'default'
+
+        """
+        return self.element_at_or_else(index, default)
+
+    
+    def element_at_or_none(self, index: int) -> Optional[T]:
+        """
+         Returns the element at the specified [index] in the Sequence or None if the index is out of bounds.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).element_at_or_none(1)
+        2
+        >>> it(lst).element_at_or_none(4) is None
+        True
+
+        """
+        return self.element_at_or_else(index, None)
+    
     def distinct(self) -> Sequence[T]:
         """
          Returns a new Sequence containing the distinct elements of the given Sequence.
@@ -869,6 +959,15 @@ class Sequence(Generic[T], Iterable[T]):
     def sum_of(self, selector: Callable[[T], float]) -> float:
         ...
     def sum_of(self, selector: Callable[[T], Union[int, float]]) -> Union[int, float]:
+        """
+         Returns the sum of the elements of the given Sequence.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).sum_of(lambda x: x)
+        6
+
+        """
         return sum(selector(i) for i in self)
     
     @overload
@@ -887,6 +986,15 @@ class Sequence(Generic[T], Iterable[T]):
     def max_of(self, selector: Callable[[T], float]) -> float:
         ...
     def max_of(self, selector: Callable[[T], Union[int, float]]) -> Union[int, float]:
+        """
+         Returns the maximum element of the given Sequence.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).max_of(lambda x: x)
+        3
+
+        """
         return max(selector(i) for i in self)
     
     @overload
@@ -907,6 +1015,126 @@ class Sequence(Generic[T], Iterable[T]):
     def min_of(self, selector: Callable[[T], Union[int, float]]) -> Union[int, float]:
         return min(selector(i) for i in self)
     
+    def mean_of(self, selector: Callable[[T], Union[int, float]]) -> Union[int, float]:
+        """
+         Returns the mean of the elements of the given Sequence.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).mean_of(lambda x: x)
+        2.0
+
+        """
+        return self.sum_of(selector) / len(self)
+    
+    @overload
+    def sum(self: Sequence[int]) -> int:
+        """
+         Returns the sum of the elements of the given Sequence.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).sum()
+        6
+
+        """
+        ...
+    @overload
+    def sum(self: Sequence[float]) -> float:
+        ...
+    def sum(self: Sequence[Union[float, int]]) -> Union[float, int]:
+        """
+         Returns the sum of the elements of the given Sequence.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).sum()
+        6
+
+        """
+        return sum(self)
+    
+    @overload
+    def max(self: Sequence[int]) -> int:
+        """
+         Returns the maximum element of the given Sequence.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).max()
+        3
+
+        """
+        ...
+    @overload
+    def max(self: Sequence[float]) -> float:
+        ...
+    def max(self: Sequence[Union[float, int]]) -> Union[float, int]:
+        """
+         Returns the maximum element of the given Sequence.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).max()
+        3
+
+        """
+        return max(self)
+    
+    @overload
+    def min(self: Sequence[int]) -> int:
+        """
+         Returns the minimum element of the given Sequence.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).min()
+        1
+
+        """
+        ...
+    @overload
+    def min(self: Sequence[float]) -> float:
+        ...
+    def min(self: Sequence[Union[float, int]]) -> Union[float, int]:
+        """
+         Returns the minimum element of the given Sequence.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).min()
+        1
+
+        """
+        return min(self)
+    
+    @overload
+    def mean(self: Sequence[int]) -> int:
+        """
+         Returns the mean of the elements of the given Sequence.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).mean()
+        2.0
+
+        """
+        ...
+    @overload
+    def mean(self: Sequence[float]) -> float:
+        ...
+    def mean(self: Sequence[Union[int, float]]) -> Union[int, float]:
+        """
+         Returns the mean of the elements of the given Sequence.
+
+         Example 1:
+        >>> lst = [1, 2, 3]
+        >>> it(lst).mean()
+        2.0
+
+        """
+        return self.sum() / len(self)
+    
 
     # noinspection PyShadowingNames
     def reversed(self) -> Sequence[T]:
@@ -1029,6 +1257,40 @@ class Sequence(Generic[T], Iterable[T]):
         (['a', 'b', 'c'], ['2'])
         """
         return self.filter(predicate), self.filter(lambda x: not predicate(x))
+    
+
+    @overload
+    def combinations(self, n: Literal[2]) -> Sequence[Tuple[T, T]]: 
+        """
+         Returns a Sequence of all possible combinations of size [n] from the given Sequence.
+
+         Example 1:
+        >>> lst = ['a', 'b', 'c']
+        >>> it(lst).combinations(2)
+        [('a', 'b'), ('a', 'c'), ('b', 'c')]
+        """
+        ...
+    
+    @overload
+    def combinations(self, n: Literal[3]) -> Sequence[Tuple[T, T, T]]: 
+        ...
+    @overload
+    def combinations(self, n: Literal[4]) -> Sequence[Tuple[T, T, T, T]]: 
+        ...
+    @overload
+    def combinations(self, n: Literal[5]) -> Sequence[Tuple[T, T, T, T, T]]: 
+        ...
+    def combinations(self, n: int) -> Sequence[Tuple[T, ...]]:
+        """
+         Returns a Sequence of all possible combinations of size [n] from the given Sequence.
+
+         Example 1:
+        >>> lst = ['a', 'b', 'c']
+        >>> it(lst).combinations(2)
+        [('a', 'b'), ('a', 'c'), ('b', 'c')]
+        """
+        return CombinationSequence(self, n)
+
 
     def to_set(self) -> Set[T]:
         """
@@ -1208,6 +1470,7 @@ class MergingSequence(Sequence[V]):
             except StopIteration:
                 break
 
+
 class GroupingSequence(Sequence[Tuple[K, List[T]]]):
     def __init__(self, iterable: Iterable[T], key_func: Callable[[T], K]) -> None:
         self._iter = iterable
@@ -1233,6 +1496,21 @@ class GroupingSequence(Sequence[Tuple[K, List[T]]]):
         yield from d.items()
 
 
+
+class CombinationSequence(Sequence[Tuple[T, ...]]):
+    def __init__(self, iterable: Iterable[T], n: int) -> None:
+        self._iter = iterable
+        self._n = n
+    
+    def __iter__(self) -> Iterator[Tuple[T, ...]]:
+        from itertools import combinations
+        yield from combinations(self._iter, self._n)
+
+
+def throw(exception: Exception) -> None:
+    raise exception
+
+
 def sequence(iterable: Iterable[T]) -> Sequence[T]:
     """
      Returns a Sequence with elements of the given Iterable.
